# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    A2ATool,
    AISearchIndexResource,
    AgentClusterInsightResult,
    AgentClusterInsightsRequest,
    AgentDefinition,
    AgentDetails,
    AgentId,
    AgentObjectVersions,
    AgentReference,
    AgentTaxonomyInput,
    AgentVersionDetails,
    AgenticIdentityCredentials,
    Annotation,
    AnnotationFileCitation,
    AnnotationFilePath,
    AnnotationUrlCitation,
    ApiErrorResponse,
    ApiKeyCredentials,
    ApproximateLocation,
    AzureAIAgentTarget,
    AzureAISearchAgentTool,
    AzureAISearchIndex,
    AzureAISearchToolResource,
    AzureFunctionAgentTool,
    AzureFunctionBinding,
    AzureFunctionDefinition,
    AzureFunctionDefinitionFunction,
    AzureFunctionStorageQueue,
    AzureOpenAIModelConfiguration,
    BaseCredentials,
    BingCustomSearchAgentTool,
    BingCustomSearchConfiguration,
    BingCustomSearchToolParameters,
    BingGroundingAgentTool,
    BingGroundingSearchConfiguration,
    BingGroundingSearchToolParameters,
    BlobReference,
    BlobReferenceSasCredential,
    BrowserAutomationAgentTool,
    BrowserAutomationToolConnectionParameters,
    BrowserAutomationToolParameters,
    CaptureStructuredOutputsTool,
    ChartCoordinate,
    ChatSummaryMemoryItem,
    ClusterInsightResult,
    ClusterTokenUsage,
    CodeBasedEvaluatorDefinition,
    CodeInterpreterOutput,
    CodeInterpreterOutputImage,
    CodeInterpreterOutputLogs,
    CodeInterpreterTool,
    CodeInterpreterToolAuto,
    CodeInterpreterToolCallItemParam,
    CodeInterpreterToolCallItemResource,
    ComparisonFilter,
    CompoundFilter,
    ComputerAction,
    ComputerActionClick,
    ComputerActionDoubleClick,
    ComputerActionDrag,
    ComputerActionKeyPress,
    ComputerActionMove,
    ComputerActionScreenshot,
    ComputerActionScroll,
    ComputerActionTypeKeys,
    ComputerActionWait,
    ComputerToolCallItemParam,
    ComputerToolCallItemResource,
    ComputerToolCallOutputItemOutput,
    ComputerToolCallOutputItemOutputComputerScreenshot,
    ComputerToolCallOutputItemParam,
    ComputerToolCallOutputItemResource,
    ComputerToolCallSafetyCheck,
    ComputerUsePreviewTool,
    Connection,
    ContainerAppAgentDefinition,
    ContinuousEvaluationRuleAction,
    Coordinate,
    CosmosDBIndex,
    CreatedBy,
    CronTrigger,
    CustomCredential,
    DailyRecurrenceSchedule,
    DatasetCredential,
    DatasetVersion,
    DeleteAgentResponse,
    DeleteAgentVersionResponse,
    DeleteMemoryStoreResult,
    Deployment,
    EmbeddingConfiguration,
    EntraIDCredentials,
    Error,
    EvalCompareReport,
    EvalResult,
    EvalRunResultCompareItem,
    EvalRunResultComparison,
    EvalRunResultSummary,
    EvaluationComparisonRequest,
    EvaluationResultSample,
    EvaluationRule,
    EvaluationRuleAction,
    EvaluationRuleFilter,
    EvaluationRunClusterInsightResult,
    EvaluationRunClusterInsightsRequest,
    EvaluationScheduleTask,
    EvaluationTaxonomy,
    EvaluationTaxonomyInput,
    EvaluatorDefinition,
    EvaluatorMetric,
    EvaluatorVersion,
    FabricDataAgentToolParameters,
    FieldMapping,
    FileDatasetVersion,
    FileSearchTool,
    FileSearchToolCallItemParam,
    FileSearchToolCallItemParamResult,
    FileSearchToolCallItemResource,
    FolderDatasetVersion,
    FunctionTool,
    FunctionToolCallItemParam,
    FunctionToolCallItemResource,
    FunctionToolCallOutputItemParam,
    FunctionToolCallOutputItemResource,
    HostedAgentDefinition,
    HourlyRecurrenceSchedule,
    HumanEvaluationRuleAction,
    ImageBasedHostedAgentDefinition,
    ImageGenTool,
    ImageGenToolCallItemParam,
    ImageGenToolCallItemResource,
    ImageGenToolInputImageMask,
    Index,
    Insight,
    InsightCluster,
    InsightModelConfiguration,
    InsightRequest,
    InsightResult,
    InsightSample,
    InsightScheduleTask,
    InsightSummary,
    InsightsMetadata,
    ItemContent,
    ItemContentInputAudio,
    ItemContentInputFile,
    ItemContentInputImage,
    ItemContentInputText,
    ItemContentOutputAudio,
    ItemContentOutputText,
    ItemContentRefusal,
    ItemParam,
    ItemReferenceItemParam,
    ItemResource,
    LocalShellExecAction,
    LocalShellTool,
    LocalShellToolCallItemParam,
    LocalShellToolCallItemResource,
    LocalShellToolCallOutputItemParam,
    LocalShellToolCallOutputItemResource,
    Location,
    LogProb,
    MCPApprovalRequestItemParam,
    MCPApprovalRequestItemResource,
    MCPApprovalResponseItemParam,
    MCPApprovalResponseItemResource,
    MCPCallItemParam,
    MCPCallItemResource,
    MCPListToolsItemParam,
    MCPListToolsItemResource,
    MCPListToolsTool,
    MCPTool,
    MCPToolAllowedTools1,
    MCPToolRequireApproval1,
    MCPToolRequireApprovalAlways,
    MCPToolRequireApprovalNever,
    ManagedAzureAISearchIndex,
    MemoryItem,
    MemoryOperation,
    MemorySearchItem,
    MemorySearchOptions,
    MemorySearchTool,
    MemorySearchToolCallItemParam,
    MemorySearchToolCallItemResource,
    MemoryStoreDefaultDefinition,
    MemoryStoreDefaultOptions,
    MemoryStoreDefinition,
    MemoryStoreDeleteScopeResult,
    MemoryStoreDetails,
    MemoryStoreOperationUsage,
    MemoryStoreOperationUsageInputTokensDetails,
    MemoryStoreOperationUsageOutputTokensDetails,
    MemoryStoreSearchResult,
    MemoryStoreUpdateCompletedResult,
    MemoryStoreUpdateResult,
    MicrosoftFabricAgentTool,
    ModelDeployment,
    ModelDeploymentSku,
    MonthlyRecurrenceSchedule,
    NoAuthenticationCredentials,
    OAuthConsentRequestItemResource,
    OneTimeTrigger,
    OpenApiAgentTool,
    OpenApiAnonymousAuthDetails,
    OpenApiAuthDetails,
    OpenApiFunctionDefinition,
    OpenApiFunctionDefinitionFunction,
    OpenApiManagedAuthDetails,
    OpenApiManagedSecurityScheme,
    OpenApiProjectConnectionAuthDetails,
    OpenApiProjectConnectionSecurityScheme,
    PendingUploadRequest,
    PendingUploadResponse,
    Prompt,
    PromptAgentDefinition,
    PromptAgentDefinitionText,
    PromptBasedEvaluatorDefinition,
    ProtocolVersionRecord,
    RaiConfig,
    RankingOptions,
    Reasoning,
    ReasoningItemParam,
    ReasoningItemResource,
    ReasoningItemSummaryPart,
    ReasoningItemSummaryTextPart,
    RecurrenceSchedule,
    RecurrenceTrigger,
    RedTeam,
    Response,
    ResponseCodeInterpreterCallCodeDeltaEvent,
    ResponseCodeInterpreterCallCodeDoneEvent,
    ResponseCodeInterpreterCallCompletedEvent,
    ResponseCodeInterpreterCallInProgressEvent,
    ResponseCodeInterpreterCallInterpretingEvent,
    ResponseCompletedEvent,
    ResponseContentPartAddedEvent,
    ResponseContentPartDoneEvent,
    ResponseConversation1,
    ResponseCreatedEvent,
    ResponseError,
    ResponseErrorEvent,
    ResponseFailedEvent,
    ResponseFileSearchCallCompletedEvent,
    ResponseFileSearchCallInProgressEvent,
    ResponseFileSearchCallSearchingEvent,
    ResponseFunctionCallArgumentsDeltaEvent,
    ResponseFunctionCallArgumentsDoneEvent,
    ResponseImageGenCallCompletedEvent,
    ResponseImageGenCallGeneratingEvent,
    ResponseImageGenCallInProgressEvent,
    ResponseImageGenCallPartialImageEvent,
    ResponseInProgressEvent,
    ResponseIncompleteDetails1,
    ResponseIncompleteEvent,
    ResponseMCPCallArgumentsDeltaEvent,
    ResponseMCPCallArgumentsDoneEvent,
    ResponseMCPCallCompletedEvent,
    ResponseMCPCallFailedEvent,
    ResponseMCPCallInProgressEvent,
    ResponseMCPListToolsCompletedEvent,
    ResponseMCPListToolsFailedEvent,
    ResponseMCPListToolsInProgressEvent,
    ResponseOutputItemAddedEvent,
    ResponseOutputItemDoneEvent,
    ResponsePromptVariables,
    ResponseQueuedEvent,
    ResponseReasoningDeltaEvent,
    ResponseReasoningDoneEvent,
    ResponseReasoningSummaryDeltaEvent,
    ResponseReasoningSummaryDoneEvent,
    ResponseReasoningSummaryPartAddedEvent,
    ResponseReasoningSummaryPartDoneEvent,
    ResponseReasoningSummaryTextDeltaEvent,
    ResponseReasoningSummaryTextDoneEvent,
    ResponseRefusalDeltaEvent,
    ResponseRefusalDoneEvent,
    ResponseStreamEvent,
    ResponseText,
    ResponseTextDeltaEvent,
    ResponseTextDoneEvent,
    ResponseTextFormatConfiguration,
    ResponseTextFormatConfigurationJsonObject,
    ResponseTextFormatConfigurationJsonSchema,
    ResponseTextFormatConfigurationText,
    ResponseUsage,
    ResponseWebSearchCallCompletedEvent,
    ResponseWebSearchCallInProgressEvent,
    ResponseWebSearchCallSearchingEvent,
    ResponsesAssistantMessageItemParam,
    ResponsesAssistantMessageItemResource,
    ResponsesDeveloperMessageItemParam,
    ResponsesDeveloperMessageItemResource,
    ResponsesMessageItemParam,
    ResponsesMessageItemResource,
    ResponsesSystemMessageItemParam,
    ResponsesSystemMessageItemResource,
    ResponsesUserMessageItemParam,
    ResponsesUserMessageItemResource,
    SASCredentials,
    Schedule,
    ScheduleRun,
    ScheduleTask,
    SharepointAgentTool,
    SharepointGroundingToolParameters,
    StructuredInputDefinition,
    StructuredOutputDefinition,
    StructuredOutputsItemResource,
    Target,
    TargetConfig,
    TaxonomyCategory,
    TaxonomySubCategory,
    Tool,
    ToolChoiceObject,
    ToolChoiceObjectCodeInterpreter,
    ToolChoiceObjectComputer,
    ToolChoiceObjectFileSearch,
    ToolChoiceObjectFunction,
    ToolChoiceObjectImageGen,
    ToolChoiceObjectMCP,
    ToolChoiceObjectWebSearch,
    ToolDescription,
    ToolProjectConnection,
    TopLogProb,
    Trigger,
    UserProfileMemoryItem,
    VectorStoreFileAttributes,
    WebSearchAction,
    WebSearchActionFind,
    WebSearchActionOpenPage,
    WebSearchActionSearch,
    WebSearchActionSearchSources,
    WebSearchPreviewTool,
    WebSearchToolCallItemParam,
    WebSearchToolCallItemResource,
    WeeklyRecurrenceSchedule,
    WorkflowActionOutputItemResource,
    WorkflowAgentDefinition,
)

from ._enums import (  # type: ignore
    AgentKind,
    AgentProtocol,
    AnnotationType,
    AttackStrategy,
    AzureAISearchQueryType,
    CodeInterpreterOutputType,
    ComputerActionType,
    ComputerToolCallOutputItemOutputType,
    ConnectionType,
    CredentialType,
    DatasetType,
    DayOfWeek,
    DeploymentType,
    EvaluationRuleActionType,
    EvaluationRuleEventType,
    EvaluationTaxonomyInputType,
    EvaluatorCategory,
    EvaluatorDefinitionType,
    EvaluatorMetricDirection,
    EvaluatorMetricType,
    EvaluatorType,
    IndexType,
    InsightType,
    ItemContentType,
    ItemType,
    LocationType,
    MemoryItemKind,
    MemoryOperationKind,
    MemoryStoreKind,
    MemoryStoreUpdateStatus,
    OpenApiAuthType,
    OperationState,
    PendingUploadType,
    ReasoningEffort,
    ReasoningItemSummaryPartType,
    RecurrenceType,
    ResponseErrorCode,
    ResponseStreamEventType,
    ResponseTextFormatConfigurationType,
    ResponsesMessageRole,
    RiskCategory,
    SampleType,
    ScheduleProvisioningStatus,
    ScheduleTaskType,
    ServiceTier,
    ToolChoiceObjectType,
    ToolChoiceOptions,
    ToolType,
    TreatmentEffectType,
    TriggerType,
    WebSearchActionType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "A2ATool",
    "AISearchIndexResource",
    "AgentClusterInsightResult",
    "AgentClusterInsightsRequest",
    "AgentDefinition",
    "AgentDetails",
    "AgentId",
    "AgentObjectVersions",
    "AgentReference",
    "AgentTaxonomyInput",
    "AgentVersionDetails",
    "AgenticIdentityCredentials",
    "Annotation",
    "AnnotationFileCitation",
    "AnnotationFilePath",
    "AnnotationUrlCitation",
    "ApiErrorResponse",
    "ApiKeyCredentials",
    "ApproximateLocation",
    "AzureAIAgentTarget",
    "AzureAISearchAgentTool",
    "AzureAISearchIndex",
    "AzureAISearchToolResource",
    "AzureFunctionAgentTool",
    "AzureFunctionBinding",
    "AzureFunctionDefinition",
    "AzureFunctionDefinitionFunction",
    "AzureFunctionStorageQueue",
    "AzureOpenAIModelConfiguration",
    "BaseCredentials",
    "BingCustomSearchAgentTool",
    "BingCustomSearchConfiguration",
    "BingCustomSearchToolParameters",
    "BingGroundingAgentTool",
    "BingGroundingSearchConfiguration",
    "BingGroundingSearchToolParameters",
    "BlobReference",
    "BlobReferenceSasCredential",
    "BrowserAutomationAgentTool",
    "BrowserAutomationToolConnectionParameters",
    "BrowserAutomationToolParameters",
    "CaptureStructuredOutputsTool",
    "ChartCoordinate",
    "ChatSummaryMemoryItem",
    "ClusterInsightResult",
    "ClusterTokenUsage",
    "CodeBasedEvaluatorDefinition",
    "CodeInterpreterOutput",
    "CodeInterpreterOutputImage",
    "CodeInterpreterOutputLogs",
    "CodeInterpreterTool",
    "CodeInterpreterToolAuto",
    "CodeInterpreterToolCallItemParam",
    "CodeInterpreterToolCallItemResource",
    "ComparisonFilter",
    "CompoundFilter",
    "ComputerAction",
    "ComputerActionClick",
    "ComputerActionDoubleClick",
    "ComputerActionDrag",
    "ComputerActionKeyPress",
    "ComputerActionMove",
    "ComputerActionScreenshot",
    "ComputerActionScroll",
    "ComputerActionTypeKeys",
    "ComputerActionWait",
    "ComputerToolCallItemParam",
    "ComputerToolCallItemResource",
    "ComputerToolCallOutputItemOutput",
    "ComputerToolCallOutputItemOutputComputerScreenshot",
    "ComputerToolCallOutputItemParam",
    "ComputerToolCallOutputItemResource",
    "ComputerToolCallSafetyCheck",
    "ComputerUsePreviewTool",
    "Connection",
    "ContainerAppAgentDefinition",
    "ContinuousEvaluationRuleAction",
    "Coordinate",
    "CosmosDBIndex",
    "CreatedBy",
    "CronTrigger",
    "CustomCredential",
    "DailyRecurrenceSchedule",
    "DatasetCredential",
    "DatasetVersion",
    "DeleteAgentResponse",
    "DeleteAgentVersionResponse",
    "DeleteMemoryStoreResult",
    "Deployment",
    "EmbeddingConfiguration",
    "EntraIDCredentials",
    "Error",
    "EvalCompareReport",
    "EvalResult",
    "EvalRunResultCompareItem",
    "EvalRunResultComparison",
    "EvalRunResultSummary",
    "EvaluationComparisonRequest",
    "EvaluationResultSample",
    "EvaluationRule",
    "EvaluationRuleAction",
    "EvaluationRuleFilter",
    "EvaluationRunClusterInsightResult",
    "EvaluationRunClusterInsightsRequest",
    "EvaluationScheduleTask",
    "EvaluationTaxonomy",
    "EvaluationTaxonomyInput",
    "EvaluatorDefinition",
    "EvaluatorMetric",
    "EvaluatorVersion",
    "FabricDataAgentToolParameters",
    "FieldMapping",
    "FileDatasetVersion",
    "FileSearchTool",
    "FileSearchToolCallItemParam",
    "FileSearchToolCallItemParamResult",
    "FileSearchToolCallItemResource",
    "FolderDatasetVersion",
    "FunctionTool",
    "FunctionToolCallItemParam",
    "FunctionToolCallItemResource",
    "FunctionToolCallOutputItemParam",
    "FunctionToolCallOutputItemResource",
    "HostedAgentDefinition",
    "HourlyRecurrenceSchedule",
    "HumanEvaluationRuleAction",
    "ImageBasedHostedAgentDefinition",
    "ImageGenTool",
    "ImageGenToolCallItemParam",
    "ImageGenToolCallItemResource",
    "ImageGenToolInputImageMask",
    "Index",
    "Insight",
    "InsightCluster",
    "InsightModelConfiguration",
    "InsightRequest",
    "InsightResult",
    "InsightSample",
    "InsightScheduleTask",
    "InsightSummary",
    "InsightsMetadata",
    "ItemContent",
    "ItemContentInputAudio",
    "ItemContentInputFile",
    "ItemContentInputImage",
    "ItemContentInputText",
    "ItemContentOutputAudio",
    "ItemContentOutputText",
    "ItemContentRefusal",
    "ItemParam",
    "ItemReferenceItemParam",
    "ItemResource",
    "LocalShellExecAction",
    "LocalShellTool",
    "LocalShellToolCallItemParam",
    "LocalShellToolCallItemResource",
    "LocalShellToolCallOutputItemParam",
    "LocalShellToolCallOutputItemResource",
    "Location",
    "LogProb",
    "MCPApprovalRequestItemParam",
    "MCPApprovalRequestItemResource",
    "MCPApprovalResponseItemParam",
    "MCPApprovalResponseItemResource",
    "MCPCallItemParam",
    "MCPCallItemResource",
    "MCPListToolsItemParam",
    "MCPListToolsItemResource",
    "MCPListToolsTool",
    "MCPTool",
    "MCPToolAllowedTools1",
    "MCPToolRequireApproval1",
    "MCPToolRequireApprovalAlways",
    "MCPToolRequireApprovalNever",
    "ManagedAzureAISearchIndex",
    "MemoryItem",
    "MemoryOperation",
    "MemorySearchItem",
    "MemorySearchOptions",
    "MemorySearchTool",
    "MemorySearchToolCallItemParam",
    "MemorySearchToolCallItemResource",
    "MemoryStoreDefaultDefinition",
    "MemoryStoreDefaultOptions",
    "MemoryStoreDefinition",
    "MemoryStoreDeleteScopeResult",
    "MemoryStoreDetails",
    "MemoryStoreOperationUsage",
    "MemoryStoreOperationUsageInputTokensDetails",
    "MemoryStoreOperationUsageOutputTokensDetails",
    "MemoryStoreSearchResult",
    "MemoryStoreUpdateCompletedResult",
    "MemoryStoreUpdateResult",
    "MicrosoftFabricAgentTool",
    "ModelDeployment",
    "ModelDeploymentSku",
    "MonthlyRecurrenceSchedule",
    "NoAuthenticationCredentials",
    "OAuthConsentRequestItemResource",
    "OneTimeTrigger",
    "OpenApiAgentTool",
    "OpenApiAnonymousAuthDetails",
    "OpenApiAuthDetails",
    "OpenApiFunctionDefinition",
    "OpenApiFunctionDefinitionFunction",
    "OpenApiManagedAuthDetails",
    "OpenApiManagedSecurityScheme",
    "OpenApiProjectConnectionAuthDetails",
    "OpenApiProjectConnectionSecurityScheme",
    "PendingUploadRequest",
    "PendingUploadResponse",
    "Prompt",
    "PromptAgentDefinition",
    "PromptAgentDefinitionText",
    "PromptBasedEvaluatorDefinition",
    "ProtocolVersionRecord",
    "RaiConfig",
    "RankingOptions",
    "Reasoning",
    "ReasoningItemParam",
    "ReasoningItemResource",
    "ReasoningItemSummaryPart",
    "ReasoningItemSummaryTextPart",
    "RecurrenceSchedule",
    "RecurrenceTrigger",
    "RedTeam",
    "Response",
    "ResponseCodeInterpreterCallCodeDeltaEvent",
    "ResponseCodeInterpreterCallCodeDoneEvent",
    "ResponseCodeInterpreterCallCompletedEvent",
    "ResponseCodeInterpreterCallInProgressEvent",
    "ResponseCodeInterpreterCallInterpretingEvent",
    "ResponseCompletedEvent",
    "ResponseContentPartAddedEvent",
    "ResponseContentPartDoneEvent",
    "ResponseConversation1",
    "ResponseCreatedEvent",
    "ResponseError",
    "ResponseErrorEvent",
    "ResponseFailedEvent",
    "ResponseFileSearchCallCompletedEvent",
    "ResponseFileSearchCallInProgressEvent",
    "ResponseFileSearchCallSearchingEvent",
    "ResponseFunctionCallArgumentsDeltaEvent",
    "ResponseFunctionCallArgumentsDoneEvent",
    "ResponseImageGenCallCompletedEvent",
    "ResponseImageGenCallGeneratingEvent",
    "ResponseImageGenCallInProgressEvent",
    "ResponseImageGenCallPartialImageEvent",
    "ResponseInProgressEvent",
    "ResponseIncompleteDetails1",
    "ResponseIncompleteEvent",
    "ResponseMCPCallArgumentsDeltaEvent",
    "ResponseMCPCallArgumentsDoneEvent",
    "ResponseMCPCallCompletedEvent",
    "ResponseMCPCallFailedEvent",
    "ResponseMCPCallInProgressEvent",
    "ResponseMCPListToolsCompletedEvent",
    "ResponseMCPListToolsFailedEvent",
    "ResponseMCPListToolsInProgressEvent",
    "ResponseOutputItemAddedEvent",
    "ResponseOutputItemDoneEvent",
    "ResponsePromptVariables",
    "ResponseQueuedEvent",
    "ResponseReasoningDeltaEvent",
    "ResponseReasoningDoneEvent",
    "ResponseReasoningSummaryDeltaEvent",
    "ResponseReasoningSummaryDoneEvent",
    "ResponseReasoningSummaryPartAddedEvent",
    "ResponseReasoningSummaryPartDoneEvent",
    "ResponseReasoningSummaryTextDeltaEvent",
    "ResponseReasoningSummaryTextDoneEvent",
    "ResponseRefusalDeltaEvent",
    "ResponseRefusalDoneEvent",
    "ResponseStreamEvent",
    "ResponseText",
    "ResponseTextDeltaEvent",
    "ResponseTextDoneEvent",
    "ResponseTextFormatConfiguration",
    "ResponseTextFormatConfigurationJsonObject",
    "ResponseTextFormatConfigurationJsonSchema",
    "ResponseTextFormatConfigurationText",
    "ResponseUsage",
    "ResponseWebSearchCallCompletedEvent",
    "ResponseWebSearchCallInProgressEvent",
    "ResponseWebSearchCallSearchingEvent",
    "ResponsesAssistantMessageItemParam",
    "ResponsesAssistantMessageItemResource",
    "ResponsesDeveloperMessageItemParam",
    "ResponsesDeveloperMessageItemResource",
    "ResponsesMessageItemParam",
    "ResponsesMessageItemResource",
    "ResponsesSystemMessageItemParam",
    "ResponsesSystemMessageItemResource",
    "ResponsesUserMessageItemParam",
    "ResponsesUserMessageItemResource",
    "SASCredentials",
    "Schedule",
    "ScheduleRun",
    "ScheduleTask",
    "SharepointAgentTool",
    "SharepointGroundingToolParameters",
    "StructuredInputDefinition",
    "StructuredOutputDefinition",
    "StructuredOutputsItemResource",
    "Target",
    "TargetConfig",
    "TaxonomyCategory",
    "TaxonomySubCategory",
    "Tool",
    "ToolChoiceObject",
    "ToolChoiceObjectCodeInterpreter",
    "ToolChoiceObjectComputer",
    "ToolChoiceObjectFileSearch",
    "ToolChoiceObjectFunction",
    "ToolChoiceObjectImageGen",
    "ToolChoiceObjectMCP",
    "ToolChoiceObjectWebSearch",
    "ToolDescription",
    "ToolProjectConnection",
    "TopLogProb",
    "Trigger",
    "UserProfileMemoryItem",
    "VectorStoreFileAttributes",
    "WebSearchAction",
    "WebSearchActionFind",
    "WebSearchActionOpenPage",
    "WebSearchActionSearch",
    "WebSearchActionSearchSources",
    "WebSearchPreviewTool",
    "WebSearchToolCallItemParam",
    "WebSearchToolCallItemResource",
    "WeeklyRecurrenceSchedule",
    "WorkflowActionOutputItemResource",
    "WorkflowAgentDefinition",
    "AgentKind",
    "AgentProtocol",
    "AnnotationType",
    "AttackStrategy",
    "AzureAISearchQueryType",
    "CodeInterpreterOutputType",
    "ComputerActionType",
    "ComputerToolCallOutputItemOutputType",
    "ConnectionType",
    "CredentialType",
    "DatasetType",
    "DayOfWeek",
    "DeploymentType",
    "EvaluationRuleActionType",
    "EvaluationRuleEventType",
    "EvaluationTaxonomyInputType",
    "EvaluatorCategory",
    "EvaluatorDefinitionType",
    "EvaluatorMetricDirection",
    "EvaluatorMetricType",
    "EvaluatorType",
    "IndexType",
    "InsightType",
    "ItemContentType",
    "ItemType",
    "LocationType",
    "MemoryItemKind",
    "MemoryOperationKind",
    "MemoryStoreKind",
    "MemoryStoreUpdateStatus",
    "OpenApiAuthType",
    "OperationState",
    "PendingUploadType",
    "ReasoningEffort",
    "ReasoningItemSummaryPartType",
    "RecurrenceType",
    "ResponseErrorCode",
    "ResponseStreamEventType",
    "ResponseTextFormatConfigurationType",
    "ResponsesMessageRole",
    "RiskCategory",
    "SampleType",
    "ScheduleProvisioningStatus",
    "ScheduleTaskType",
    "ServiceTier",
    "ToolChoiceObjectType",
    "ToolChoiceOptions",
    "ToolType",
    "TreatmentEffectType",
    "TriggerType",
    "WebSearchActionType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
